﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using OpenTK;
using OpenTK.Graphics;
using OpenTK.Graphics.OpenGL;

using Meshomatic;

namespace CG1.Ex06.Meshes
{
    class Mesh
    {
        #region Properties

        private MeshBatch MBatch;
        private MeshData MappedData, DefaultData;
        private Random Rand = new Random();
        private String objFile, texFile;

        #endregion

        #region Init

        /// <summary>
        /// Set the Mesh. Here you can edit mesh, or improve to better performance if needed.
        /// </summary>
        public Mesh(String ObjFile, String TexFile)
        {
            objFile = ObjFile;
            texFile = TexFile;

            MBatch = new MeshBatch(ObjFile, TexFile);
            MappedData = new ObjLoader().LoadFile(ObjFile);
            for (int i = 0; i < MappedData.Vertices.Length; i++)
            {
                double x = MappedData.Vertices[i].X;
                double y = MappedData.Vertices[i].Y;
                double z = MappedData.Vertices[i].Z;

                MappedData.Vertices[i].X = x * Math.Sqrt(1 - y * y / 2 - z * z / 2 + y * y * z * z / 3);
                MappedData.Vertices[i].Y = y * Math.Sqrt(1 - x * x / 2 - z * z / 2 + x * x * z * z / 3);
                MappedData.Vertices[i].Z = z * Math.Sqrt(1 - x * x / 2 - y * y / 2 + x * x * y * y / 3);
            }
            DefaultData = new ObjLoader().LoadFile(ObjFile);
        }

        #endregion

        #region Edit Functions

        //ToDo: Create two simple functions. First to simple edit vertices of mesh. Second one to create some sort of morphing between two models - cube and sphere. 
        //      You can choose what you want(if similar). Change GUI if needed. Change structure if needed. Create new methods, functions if needed.
        public void randomEdit() {
            Random r = new Random();
        }

        //Hint: From sample, first function will simply scale each vertex with some random factor (Remember - vertices are also vectors).
        //      From sample, second function uses two different MeshData 'MappedData and DefaultData'. 
        //      At first we will map our mesh from cube to sphere - How? http://goo.gl/3kHHK - works only for 1's sphere, but it is enough. - MappedData is for this mapping.
        //      Then we will interpolate between DefaultData and MappedData. Interpolation will be controlled from app GUI using a trackbar.
        public void sphereTransform(int interpolation)
        {
            double inter = (double)interpolation / (double)100;
        }


        #endregion

        #region Render And Helper Methods

        /// <summary>
        /// Reload buffer of mesh and render it again.
        /// </summary>
        public void Render()
        {
            MBatch.LoadBuffers(MBatch.m);

            GL.PushMatrix();
            MBatch.DrawBuffer();
            GL.PopMatrix();
        }

        /// <summary>
        /// Set a default. Here this means: load a default data structure one more time.
        /// </summary>
        public void SetDefault()
        {
            MBatch.m = new ObjLoader().LoadFile(objFile);
        }

        #endregion
    }
}
